/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.notification.misc
{

import flash.display.DisplayObject;
import flash.events.Event;
import flash.events.HTTPStatusEvent;
import flash.events.IOErrorEvent;
import flash.events.ProgressEvent;
import flash.events.SecurityErrorEvent;
import flash.net.FileReference;
import flash.net.URLLoader;
import flash.net.URLRequest;
import flash.net.URLRequestMethod;
import flash.net.URLVariables;

import mx.controls.Alert;
import mx.core.Application;
import mx.core.FlexGlobals;
import mx.core.mx_internal;
import mx.events.CloseEvent;
import mx.managers.PopUpManager;

import org.ow2.jasmine.monitoring.eos.common.controls.ProgressBox;
import org.ow2.jasmine.monitoring.eos.common.utils.HttpRequestURLFactory;

public class DownloadFileManager
{
    public function DownloadFileManager()
    {
    }
        
    // =========================================================================
    // Properties
    // =========================================================================

    /**
    * Progress bar
    */
    private var progress:ProgressBox;
    
    /**
    * FileReference to download the exported file
    */
    private var fileRef:FileReference = new FileReference();
    
    /**
    * The message displayed when the file is ready to download
    */
    private var readyToDownloadMessage : Alert;
    
    /**
    * The request made to download the file after it's ready
    */
    private var downloadRequest : URLRequest;
    
    // Assets
        
    /**
    * Export icon (48 pixels)
    */
    [Embed(source='/../assets/export48.png')]
    [Bindable]
    public var exportIcon48:Class;
    
        
    // =========================================================================
    // Public static methods
    // =========================================================================
    
    /**
    * Download the given file.
    */
    public static function downloadFile(file:String):void
    {
        // Instanciate the manager
        var instance : DownloadFileManager = new DownloadFileManager();
        
        // Export the packages
        instance.downloadFromFile(file);
    }
    
    
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Download the file.
    */
    private function downloadFromFile(file:String):void
    {
        // Prepare the request
        var exportRequest:URLRequest = new URLRequest();

        // Build the request data
        var requestData : URLVariables = new URLVariables("file=" + file);
        
        // Setup the request
        exportRequest.data = requestData;
        exportRequest.url = HttpRequestURLFactory.getServletURL("/PrepareDownloadFileServlet");
        exportRequest.method = URLRequestMethod.POST;
        
        // Launch the request and listen to events
        var urlLoader:URLLoader = new URLLoader();
        urlLoader.addEventListener(Event.COMPLETE, completeHandler);
        urlLoader.addEventListener(ProgressEvent.PROGRESS, progressHandler);
        urlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
        urlLoader.addEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusHandler);
        urlLoader.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        urlLoader.load(exportRequest);
        
        // Display a progress box
        progress = PopUpManager.createPopUp(FlexGlobals.topLevelApplication as DisplayObject, ProgressBox) as ProgressBox;
        PopUpManager.centerPopUp(progress);
    }
    
    /**
    * When the preparation is complete
    */
    private function completeHandler(event:Event):void
    {
        // Remove the progress box popup
        PopUpManager.removePopUp(progress);
        
        // Retrieve the servlet response
        var loader:URLLoader = URLLoader(event.target);
        var response : XML = XML(loader.data.toString());
        var name:String = response.file.@name;
        var size:String = response.file.@size;
        var path:String = response.file.@path;
        
        // Setup the "download file" request
        var req:String = HttpRequestURLFactory.getServletURL("/DownloadFileServlet?fileId=" + path);
        downloadRequest = new URLRequest(req);
        
        // Ask the user for its permission to download the file
        readyToDownloadMessage =
            Alert.show('Do you want to save the file ' + name + ' (' + size + 'kb) to your hard drive ?',
                       'The file is ready to be downloaded',
                       (Alert.YES | Alert.NO),
                       null,
                       readyToDownloadMessageResponseHandler,
                       exportIcon48,
                       Alert.OK);
                       
        // A hack to use htmlText with an Alert
        readyToDownloadMessage.mx_internal::alertForm.mx_internal::textField.htmlText =
            "Do you want to save the file <i>" + name + "</i> (" + size + "kb) to your hard drive ?";
    }
    
    /**
    * Handles the user response to the "ready to download" message
    */
    private function readyToDownloadMessageResponseHandler(event:CloseEvent):void
    {
        if (event.detail == Alert.YES) {
            fileRef.download(downloadRequest);
        }
    }
    
    /**
    * Handles export progress events
    */
    private function progressHandler(event:ProgressEvent):void
    {
        // Update the progress bar informations
        progress.progressBar.setProgress(event.bytesLoaded, event.bytesTotal);
        progress.progressBar.label = (event.bytesLoaded / event.bytesTotal * 100).toFixed() + "%";
    }
    
    /**
    * Handles export security error events
    */
    private function securityErrorHandler(event:SecurityErrorEvent):void
    {
        // Remove the progress box popup
        PopUpManager.removePopUp(progress);
        
        // Display a message
        Alert.show("securityErrorHandler: " + event.toString());
    }
    
    /**
    * Handles HTTP status change events
    */    
    private function httpStatusHandler(event:HTTPStatusEvent):void
    {
        // Remove the progress box popup
        PopUpManager.removePopUp(progress);
        
        // Display a message
        //Alert.show("httpStatusHandler: " + event.toString());
    }
    
    /**
    * Handles export I/O error events
    */ 
    private function ioErrorHandler(event:IOErrorEvent):void
    {
        // Remove the progress box popup
        PopUpManager.removePopUp(progress);
        
        // Display a message
        Alert.show("ioErrorHandler: " + event.toString());
    }

}
}